% script file room.m
% temperature distribution in a room
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all; close all;

% domain: a square [a,b]^2
a = 0; b = 5;

% number of grid points 
% (numbering 1...n INCLUDING the boundary points)
n = 81;


% temperature at the heaters
heaterA = 30;
heaterB = 40;
% temperature at the window
window = 15;

% generate the domain
x = linspace(a,b,n)';
y = linspace(a,b,n)';

% step size 
h = x(2)-x(1);

% generate the right hand side
[X,Y] = ndgrid(x,y);

F = zeros((n-2)^2,1);

% generate the boundary conditions
% number all boundary points anti-clockwise
% starting from lower left corner

% vector of coordinates of the boundary points:
bdx = [x; b*ones(n-2,1); x(n:-1:1); a*ones(n-2,1)];
bdy = [a*ones(n,1); y(2:n); b*ones(n-1,1); y(n-1:-1:2)];

% initialize the boundary condition by the wall temperature
% of 20
ud = 20*ones(size(bdx));

% find the window
window_index = find((bdx<=4.5)&(bdx>=1.5));
ud(window_index) = window;

% heater A
heaterA_index = find(((bdy<=4.5)&(bdy>=3))&(bdx==5));
% add n-1 to index as it is the right edge
ud(heaterA_index) = heaterA;


% heater B
heaterB_index1 = find(((bdx<=0.5)&(bdx>=0))&(bdy==5));
ud(heaterB_index1) = heaterB;
heaterB_index2 = find(((bdy<=5)&(bdy>=4))&(bdx==0));
ud(heaterB_index2) = heaterB;



% generate the matrices for Poisson equation
% with included boundary points
A = PoissonMatrix_Dirichlet(h,n);
rhs = PoissonRHS_Dirichlet(h,n,F,ud);

U = A\rhs;

% rearragne the numerical solution
Uins = U(1:(n-2)^2); % the inner grid points
Ubd = U((n-2)^2+1:end);

% stack together
UU = zeros(n,n);
UU(2:n-1,2:n-1) = reshape(Uins,[n-2,n-2]);

UU(1:n,1) = Ubd(1:n);
UU(n,2:n) = Ubd(n+1:2*n-1);
UU(n-1:-1:1,n) = Ubd(2*n:3*n-2);
UU(1,n-1:-1:2) = Ubd(3*n-1:end);



% visualise the solution
u = reshape(UU,[n,n]);
contourf(X,Y,u,n);
shading flat;
colorbar
axis square
axis equal

